unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Spin, asa_mmf, Grids, ComCtrls, Buttons;

//     Example 1
//   Data file contains DataCount records of type TDataRec.
// We fill records and then we sort them.
// Thus we work not with a file and with the ordinary pointer
// on an array of a type - array [0..DataCount-1] of TDataRec.
//   Button "Sort" - sorting array in current thread.
//   Button "Sort in thread" - create new thread, open another
// TasaMMF component on the same Data file and we sorting
// data with him.


const
  DataCount = 128;

type
  TDataRec = record
    A,B: LongInt;
    S: String[23];
  end;
  TDataArray = array[0..(DataCount-1)] of TDataRec;
  PDataArray = ^TDataArray;
  //
  TForm1 = class(TForm)
    asaMMF1: TasaMMF;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    DrawGrid1: TDrawGrid;
    ButtonSort: TButton;
    ButtonOnOff: TSpeedButton;
    asaMMF2: TasaMMF;
    ButtonThreadSort: TButton;
    procedure StringGrid1SelectCell(Sender: TObject; Col, Row: Integer;
      var CanSelect: Boolean);
    procedure StringGrid1GetEditText(Sender: TObject; ACol, ARow: Integer;
      var Value: String);
    procedure StringGrid1SetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure DrawGrid1DrawCell(Sender: TObject; Col, Row: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure ButtonSortClick(Sender: TObject);
    procedure ButtonOnOffClick(Sender: TObject);
    procedure ButtonThreadSortClick(Sender: TObject);
  private
    A: PDataArray;
    procedure OnExample1ThreadTerminate(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure QuickSort(const MMF: TasaMMF; A: PDataArray; L, R: Integer);
  function  Compare(A: PDataArray; I1, I2: Integer): Integer;
  begin
    Result := 0;
    if A^[I1].S > A^[I2].S then
      Result := 1
    else if A^[I1].S < A^[I2].S then
      Result := -1
  end;

var
  I, J: Integer;
  P: Integer;
  T: TDataRec;
begin
  repeat
    I := L;
    J := R;
    P := (L + R) shr 1;
    repeat
      while Compare(A,I, P) < 0 do Inc(I);
      while Compare(A,J, P) > 0 do Dec(J);
      if I <= J then
      begin
        MMF.WaitForSynchroObject;
        T := A^[I];
        A^[I] := A^[J];
        A^[J] := T;
        MMF.ReleaseSynchroObject;
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then QuickSort(MMF, A, L, J);
    L := I;
  until I >= R;
end;

procedure TForm1.StringGrid1SelectCell(Sender: TObject; Col, Row: Integer;
  var CanSelect: Boolean);
begin
  CanSelect := Row > 0;
end;

procedure TForm1.StringGrid1GetEditText(Sender: TObject; ACol,
  ARow: Integer; var Value: String);
begin
  Value := '';
  if not asaMMF1.Active then exit;
  if ARow = 0 then
  else begin
    case ACol of
      0: begin
        Value := IntToStr(A^[ARow-1].A);
      end;
      1: begin
        Value := IntToStr(A^[ARow-1].B);
      end;
      2: begin
        Value := A^[ARow-1].S;
      end;
    end;
  end;
end;

procedure TForm1.StringGrid1SetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: String);
begin
  if not asaMMF1.Active then exit;
  //
  if ARow = 0 then
  else begin
    case ACol of
      0: begin
        try
          asaMMF1.WaitForSynchroObject;
          A^[ARow-1].A := StrToInt(Value);
          asaMMF1.ReleaseSynchroObject;
        except
          asaMMF1.ReleaseSynchroObject;
        end;
      end;
      1: begin
        try
          asaMMF1.WaitForSynchroObject;
          A^[ARow-1].B := StrToInt(Value);
          asaMMF1.ReleaseSynchroObject;
        except
          asaMMF1.ReleaseSynchroObject;
        end;
      end;
      2: begin
        try
          asaMMF1.WaitForSynchroObject;
          A^[ARow-1].S := Value;
          asaMMF1.ReleaseSynchroObject;
        except
          asaMMF1.ReleaseSynchroObject;
        end;
      end;
    end;
  end;
end;

procedure TForm1.DrawGrid1DrawCell(Sender: TObject; Col, Row: Integer;
  Rect: TRect; State: TGridDrawState);
var
  S: String;
const
  Titles: array[0..2] of string = ('Int 1','Int 2','String');
begin
  if Row = 0 then
    S := Titles[Col]
  else
    StringGrid1GetEditText(Self,Col,Row,S);
  //
  if (Row > 0) and asaMMF1.Active and (Col=0) and (A^[Row-1].A < 0) then
    DrawGrid1.Canvas.Font.Color := clRed
  else
    DrawGrid1.Canvas.Font.Color := clBlack;
  DrawGrid1.Canvas.TextOut(Rect.Left+2,Rect.Top+2,S);
end;

procedure TForm1.ButtonSortClick(Sender: TObject);
begin
  QuickSort(asaMMF1,A,0,127);
  DrawGrid1.Refresh;
end;

procedure TForm1.ButtonOnOffClick(Sender: TObject);
begin
  try
    asaMMF1.Active := ButtonOnOff.Down;
  except
    ButtonOnOff.Down := asaMMF1.Active;
    raise;
  end;
  //
  ButtonSort.Enabled := asaMMF1.Active;
  ButtonThreadSort.Enabled := asaMMF1.Active;
  DrawGrid1.Enabled := asaMMF1.Active;
  //
  if asaMMF1.Active then begin
    ButtonOnOff.Caption := 'Off';
    pointer(A) := asaMMF1.Data;
    DrawGrid1.Refresh;
  end else begin
    if DrawGrid1.EditorMode then
      DrawGrid1.EditorMode := False;
    ButtonOnOff.Caption := 'On';
  end;
end;

type
  TExample1Thread = class(TThread)
  public
    MMF: TasaMMF;
    procedure Execute; override;
  end;

procedure TExample1Thread.Execute;
begin
  if (MMF = nil) or not MMF.Active then exit;
  Sleep(5000);
  QuickSort(MMF,PDataArray(MMF.Data),0,127);
end;

var
  Example1Thread: TExample1Thread;

procedure TForm1.OnExample1ThreadTerminate(Sender: TObject);
begin
  asaMMF2.Active := False;
  ButtonSort.Enabled := True;
  ButtonThreadSort.Enabled := True;
  DrawGrid1.Refresh;
end;

procedure TForm1.ButtonThreadSortClick(Sender: TObject);
begin
  asaMMF2.Active := True;
  Example1Thread := TExample1Thread.Create(True);
  Example1Thread.MMF := asaMMF2;
  Example1Thread.OnTerminate := OnExample1ThreadTerminate;
  ButtonSort.Enabled := False;
  ButtonThreadSort.Enabled := False;
  Example1Thread.Resume;
end;

end.
